package com.rma.epstopik_bengaliword;

import android.app.ProgressDialog;
import android.content.Context;
import android.os.AsyncTask;
import android.content.DialogInterface;
import android.util.Log;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.ref.WeakReference;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import androidx.appcompat.app.AlertDialog;
import com.rma.epstopik_bengaliword.PdfDownloader.DownloadListener;

public class DownloadHelper {

    final WeakReference<Context> contextRef;
    final String URL;
    final String fileName;
    final DownloadListener Listener;
    DownloadTask downloadTask;

    public DownloadHelper(Context context, String URL, String fileName, DownloadListener Listener) {
        this.contextRef = new WeakReference<>(context);
        this.URL = URL;
        this.fileName = fileName;
        this.Listener = Listener;
    }

    public boolean isValidURL() {
        try {
            new URL(URL);
            return true;
        } catch (MalformedURLException e) {
            e.printStackTrace();
            Log.e("DownloadHelper", "Invalid URL: " + e.getMessage());
            return false;
        }
    }

    public void startDownload() {
        if (!isValidURL()) {
            if (Listener != null) {
                Listener.onDownloadFailed();
            }
            return; // Exit the method if URL is invalid
        }
        try {
            downloadTask = new DownloadTask(contextRef, URL, fileName, Listener);
            downloadTask.executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
        } catch (Exception e) {
            e.printStackTrace();
            Log.e("DownloadHelper", "Error starting download: " + e.getMessage());
            if (Listener != null) {
                Listener.onDownloadFailed();
            }
        }
    }

    private static class DownloadTask extends AsyncTask<Void, Integer, Boolean> {

        private final WeakReference<Context> contextRef;
        private final String URL;
        private final String fileName;
        private final DownloadListener Listener;
        private ProgressDialog dialog;

        public DownloadTask(WeakReference<Context> contextRef, String URL, String fileName, DownloadListener Listener) {
            this.contextRef = contextRef;
            this.URL = URL;
            this.fileName = fileName;
            this.Listener = Listener;
        }

        @Override
        protected void onPreExecute() {
            super.onPreExecute();
            Context context = contextRef.get();
            if (context == null) {
                cancel(true);
                return;
            }
            dialog = new ProgressDialog(context);
            dialog.setTitle("Downloading...");
            dialog.setMessage("Please wait a moment.....");
            dialog.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
            dialog.setCancelable(false);
            dialog.setButton(DialogInterface.BUTTON_NEGATIVE, "Cancel", (dialog, which) -> cancel(true));
            dialog.show();
        }

        @Override
        protected Boolean doInBackground(Void... voids) {
            Context context = contextRef.get();
            if (context == null) {
                return false;
            }

            File directory = new File(context.getExternalFilesDir(null), context.getString(R.string.app_name));

            if (!directory.exists()) {
                directory.mkdirs();
            }

            File pdfFile = new File(directory, fileName);

            try {
                URL url = new URL(URL);
                HttpURLConnection connection = (HttpURLConnection) url.openConnection();
                connection.setRequestMethod("GET");
                connection.connect();

                InputStream inputStream = connection.getInputStream();
                FileOutputStream fileOutputStream = new FileOutputStream(pdfFile);

                byte[] buffer = new byte[4096];
                long total = 0;
                int len;
                while ((len = inputStream.read(buffer)) != -1) {
                    total += len;
                    fileOutputStream.write(buffer, 0, len);
                    publishProgress((int) (total * 100 / connection.getContentLength()));
                    if (isCancelled()) {
                        fileOutputStream.close();
                        inputStream.close();
                        if (pdfFile.exists()) {
                            pdfFile.delete();
                        }
                        return false;
                    }
                }

                fileOutputStream.close();
                inputStream.close();
                return true;

            } catch (IOException e) {
                e.printStackTrace();
                Log.e("DownloadHelper", "Error downloading file: " + e.getMessage());
                if (pdfFile.exists()) {
                    pdfFile.delete();
                }
                return false;
            }
        }

        @Override
        protected void onPostExecute(Boolean result) {
            super.onPostExecute(result);
            if (dialog != null && dialog.isShowing()) {
                dialog.dismiss();
            }

            Context context = contextRef.get();
            if (context != null && !result) {
                new AlertDialog.Builder(context)
                        .setTitle("Download Error")
                        .setMessage("Failed to download file.")
                        .setPositiveButton("OK", (dialog, which) -> {})
                        .show();
            }

            if (result) {
                Listener.onDownloadComplete();
            } else {
                Listener.onDownloadFailed();
            }
        }

        @Override
        protected void onProgressUpdate(Integer... values) {
            super.onProgressUpdate(values);
            if (dialog != null && dialog.isShowing()) {
                dialog.setProgress(values[0]);
            }
        }
    }
}
